/* IllegalFormatPrecisionException.java
   Copyright (C) 2005  Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */


package java.util;

/** 
 * Thrown when the specified precision for a {@link Formatter}
 * argument is illegal.  This may be because the number is
 * a negative number (other than -1), the argument does not
 * accept a precision or for some other reason.
 *
 * @author Tom Tromey (tromey@redhat.com)
 * @author Andrew John Hughes (gnu_andrew@member.fsf.org)
 * @since 1.5 
 */
public class IllegalFormatPrecisionException 
  extends IllegalFormatException
{
  private static final long serialVersionUID = 18711008L;

  /**
   * The illegal precision value.
   *
   * @serial the illegal precision.
   */
  // Note: name fixed by serialization.
  private int p;

  /**
   * Constructs a new <code>IllegalFormatPrecisionException</code>
   * for the precision, <code>p</code>.
   *
   * @param p the illegal precision.
   */
  public IllegalFormatPrecisionException(int p)
  {
    super("The precision, " + p + ", is illegal.");
    this.p = p;
  }

  /**
   * Returns the illegal precision.
   *
   * @return the illegal precision.
   */
  public int getPrecision()
  {
    return p;
  }
}
